//----------------------------------------------------------------------------
// * Data Set Initialization {data_set_init_name}
//----------------------------------------------------------------------------
//	Initializes a data set with a name set to label.
//----------------------------------------------------------------------------
template<class Type_X>
Data_Set<Type_X, int>::Data_Set(std::string label){
	data_label = label;
}

//----------------------------------------------------------------------------
// * Data Set Initialization {data_set_init_name_cstring}
//----------------------------------------------------------------------------
//	Initializes a data set with a name set to label.
//----------------------------------------------------------------------------
template<class Type_X>
Data_Set<Type_X, int>::Data_Set(const char label[]){
	data_label = label;
}


//----------------------------------------------------------------------------
// * Data Set Initialization {data_set_init_both}
//----------------------------------------------------------------------------
//	Initializes a data set with a name set to label and description set to
//	info.
//----------------------------------------------------------------------------
template<class Type_X>
Data_Set<Type_X, int>::Data_Set(std::string label, std::string info){
	data_label = label;
	data_description = info;
}

//----------------------------------------------------------------------------
// * Data Set Initialization {data_set_init_both_cstring}
//----------------------------------------------------------------------------
//	Initializes a data set with a name set to label and description set to
//	info.
//----------------------------------------------------------------------------
template<class Type_X>
Data_Set<Type_X, int>::Data_Set(const char label[], const char info[]){
	data_label = label;
	data_description = info;
}
 
//----------------------------------------------------------------------------
// * Get Data Label {data_set_name}
//----------------------------------------------------------------------------
//	Returns the name of the data set.
//----------------------------------------------------------------------------
template<class Type_X>
std::string Data_Set<Type_X, int>::name() const{
	return data_label;
}

//----------------------------------------------------------------------------
// * Get Data Description {data_set_description}
//----------------------------------------------------------------------------
//	Returns the description of the data set.
//----------------------------------------------------------------------------
template<class Type_X>
std::string Data_Set<Type_X, int>::description() const{
	return data_description;
}

//----------------------------------------------------------------------------
// * Calculate Mean {data_set_mean_spec}
//----------------------------------------------------------------------------
//	Determines the mean for a single x-value of a 2-dimensional data
//	array.
//----------------------------------------------------------------------------
template<class Type_X>
double Data_Set<Type_X, int>::mean(Type_X x){
	double average = 0;

	if(data.find(x) != data.end()){
		double n = data[x].size();
		std::vector<int>::const_iterator y;

		for(y = data[x].begin(); y < data[x].end(); y++){
			average += static_cast<double>(*y);
		}
	
		average /= n;
	}

	return average;
}

//----------------------------------------------------------------------------
// * Calculate Standard Deviation {data_set_stdev_spec}
//----------------------------------------------------------------------------
//	Determines the standard deviation or standard error for a single
//	x-value of a 2-dimensional data array.
//----------------------------------------------------------------------------
template<class Type_X>
double Data_Set<Type_X, int>::stdev(Type_X x){
	double standard_error = 0;

	if(data.find(x) != data.end()){
		double average = mean(x);
		double n = data[x].size();
		std::vector<int>::const_iterator y;

		for(y = data[x].begin(); y < data[x].end(); y++){
			standard_error += (average - (*y)) * (average - (*y));
		}

		standard_error /= n;
		standard_error = sqrt(standard_error);
	}

	return standard_error;
}

//----------------------------------------------------------------------------
// * Add Data Pair {data_set_add_point_spec}
//----------------------------------------------------------------------------
//	Adds an x-y pair to a 2-dimensional data array. Each x-value may have
//	any number of corresponding y-value measurements.
//----------------------------------------------------------------------------
template<class Type_X>
void Data_Set<Type_X, int>::add_point(Type_X x, int y){
	data[x].push_back(y);
}

//----------------------------------------------------------------------------
// * Reset Data {data_set_reset_data_spec}
//----------------------------------------------------------------------------
//	Clears all keys and data vectors leaving an empty data set.
//----------------------------------------------------------------------------
template<class Type_X>
void Data_Set<Type_X, int>::reset(){
	data.clear();
}
