#ifndef PIF_DATA_SET_H
#define PIF_DATA_SET_H

#include <math.h>
#include <vector>
#include <map>
#include <string>

//----------------------------------------------------------------------------
// ** Data Set
//----------------------------------------------------------------------------
//	Odd error with templated iterator. Using specialized class only.
//----------------------------------------------------------------------------
template <class Type_X, class Type_Y>
	class Data_Set;

//----------------------------------------------------------------------------
// ** Data Set (Specialize to Integer)
//----------------------------------------------------------------------------
//	Specialized 2-dimensional data array which contains a set of 
//	y-measurements (as integers) by some key (Type_X).
//----------------------------------------------------------------------------
template<class Type_X>
class Data_Set<Type_X, int>{
public:
	Data_Set(){};
	Data_Set(std::string);
	Data_Set(std::string, std::string);
	Data_Set(const char[]);
	Data_Set(const char[], const char[]);

	std::string name() const;
	std::string description() const;

	double mean(Type_X);
	double stdev(Type_X);
	double confidence(Type_X);

	void add_point(Type_X, int);
	void reset();

private:
	std::string data_label;
	std::string data_description;
	std::map<Type_X, std::vector<int> > data;
};

#include "data_set.cpp"
#endif
