//----------------------------------------------------------------------------
// ** Simulation
//----------------------------------------------------------------------------
//	This class handles the simulation procedure and record keeping,
//	allowing experimental runs of varying numbers of computations by 
//	fault-sweeps of a constant network size or scale tests at a constant
//	fault-rate.
//----------------------------------------------------------------------------
#ifndef SIMULATION_H
#define SIMULATION_H

#include <iostream>
#include <fstream>
#include <string>
#include <cstdlib>
#include "data_set.h"
#include "tree.h"

enum schedulers{Synchronous, Power_Set, Interleaving};

class Simulation{

static const int COMPS = 1000;

public:
	Simulation();
	Simulation(schedulers);
	Simulation(const Simulation&);
	~Simulation();

	bool excluding() const;
	std::string scheduler() const;

	void set_method(schedulers);
	void set_size(unsigned int);
	void reset_data();
	void toggle_exclude();
	std::vector<const Node*> get_enabled() const;

	std::string perturb(unsigned int);

	void propagate();
	int compute(unsigned int, bool scale_test = false);
	void run();
	void run(unsigned int, unsigned int);
	void run(unsigned int);
	void run(unsigned int, unsigned int, unsigned int);
	void run_scale(unsigned int);
	void run_scale(unsigned int, unsigned int, unsigned int, unsigned int);
	void run_scale(unsigned int, unsigned int, unsigned int, unsigned int, unsigned int);

	void print_results(bool header = true) const;
	void print_results(std::ostream&, bool header = true) const;

private:
	Tree* network;

	schedulers method;
	bool exclude_neighbors;

	Data_Set<unsigned int, int> *tree_heights;
	Data_Set<unsigned int, int> *leaf_counts;
	Data_Set<unsigned int, int> *transitions;
	Data_Set<unsigned int, int> *actions;
	Data_Set<unsigned int, int> *causality;
	std::map<std::string, Data_Set<unsigned int, int>*> overhead;

	std::vector<Node*> get_enabled();
};

#include "simulation.cpp"
#endif
