//--------------------------------------------------------
// ** PIF Tree
//--------------------------------------------------------
#ifndef PIF_TREE_H
#define PIF_TREE_H

#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <deque>
#include "node.h"
#include "prufer.h"
#include "data_set.h"

class Tree{
public:
	Tree();
	Tree(unsigned int);
	Tree(const Prufer&);
	Tree(const Tree&);

	void generate();
	void generate(unsigned int);
	void generate(const Prufer&);

	void generate_unrooted();
	void generate_unrooted(unsigned int);
	void generate_unrooted(const Prufer&);

	unsigned int size() const;
	bool empty() const;
	bool rooted() const;
	Node* root();
	const Node* root() const;
	unsigned int height() const;
	unsigned int leaf_count() const;
	int max_causality() const;
	int total_actions() const;
	bool overhead(const Node*) const;
	bool legitimate() const;

	std::vector<Node>::iterator begin();
	std::vector<Node>::const_iterator begin() const;
	std::vector<Node>::iterator end();
	std::vector<Node>::const_iterator end() const;

	void stop();
	void reset_actions();
	bool set_root(int);
	void set_root(Node*);
	void update_causality(Node*);
	void unroot();

	Node* operator[](int);
	const Node* operator[](int) const;
	Tree* operator=(const Tree&);

	void print() const;
	void print(std::ostream&) const;

	friend std::ostream& operator<<(std::ostream&, const Tree&);

private:
	std::vector<Node> nodes;
	Node* data_root;

	void decode(const Prufer&);
	void add_edge(int, int);
	void add_edge(Node*, Node*);
	void create_hierarchy(Node*);
	bool legitimize(Node*);
	void define_actions();

	bool RP(const std::deque<const Node*>&) const;
	bool RP(const std::deque<const Node*>&, int) const;
	bool RQ(const std::deque<const Node*>&) const;
	bool RQ(const std::deque<const Node*>&, int, int) const;
};

#include "tree.cpp"
#endif
